package it.unimi.dsi.pixelmanipulator.controller;

//import java.awt.Label;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import it.unimi.dsi.pixelmanipulator.model.PixelModel;
import it.unimi.dsi.pixelmanipulator.utils.GenericExercise;
import it.unimi.dsi.pixelmanipulator.views.CompressedView;
import it.unimi.dsi.pixelmanipulator.views.EditableGraphicalView;
import it.unimi.dsi.pixelmanipulator.views.GraphicalView;
import it.unimi.dsi.pixelmanipulator.views.RectangleView;
import it.unimi.dsi.pixelmanipulator.views.TableView;
import it.unimi.dsi.pixelmanipulator.views.View;

public class PixelController extends Controller {

    public static final int BW = 0;
    public static final int GRAYSCALE = 1;
    public static final int RGB6 = 2;
    public static final int RGB = 3;

    /**
     * @uml.property name="model"
     * @uml.associationEnd multiplicity="(1 1)"
     */
    PixelModel model = null;
    /**
     * @uml.property name="views"
     * @uml.associationEnd multiplicity="(0 -1)"
     *                     elementType="java.util.Observer"
     */
    ArrayList<View> views = new ArrayList<View>();
    final private Shell mainShell;
    /**
     * @uml.property name="width"
     */
    private int width = 200;
    /**
     * @uml.property name="height"
     */
    private int height = 200;
    private Shell shell;

    public PixelController(PixelModel new_model, int mode, Shell parent) {
		mainShell = parent;
		shell = new Shell(parent, SWT.CLOSE);
		initShell();
		if (new_model != null) {
		    model = new_model;
		} else {
		    model = new PixelModel();
		}
	
		run();
		switch (mode) {
			case BW:
			    views.add(new CompressedView(model, shell, this));
			default:
			    views.add(new EditableGraphicalView(model, shell, this, mode));
			    //views.add(new TableView(shell, model, true, this, mode));
			    views.add(new RectangleView(model, shell, this, mode));
			    break;
		}
		Iterator<View> iterator = views.iterator();
		while (iterator.hasNext()) {
		    model.addObserver(iterator.next());
		}
		Image img = new Image(Display.getDefault(), 8, 10);
		ImageData data = img.getImageData();
		for (int x = 0; x < data.width; x++) {
		    for (int y = 0; y < data.height; y++) {
		    	data.setPixel(x, y, data.palette.getPixel(new RGB(0, 0, 0)));
		    }
		}
		model.setImage(data);

    }

    public PixelController(PixelModel new_model, int mode, Shell parent,
			   final GenericExercise ex) {
		final HashMap<String, String> proprieta = ex.getFeatures();
		mainShell = parent;
		shell = new Shell(parent, SWT.CLOSE);
		shell.setLayout(new FillLayout());
		initShell();
		if (new_model != null) {
		    model = new_model;
		} else {
		    model = new PixelModel();
		}
		shell.setText(proprieta.get("track"));
		Text descrizione = new Text(shell, SWT.FILL);
		descrizione.setEditable(false);
		descrizione.setText(proprieta.get("description"));
		Button consegna = new Button(shell, SWT.PUSH);
		consegna.setText("Consegna");
		consegna.addSelectionListener(new SelectionAdapter() {
			@Override
			    public void widgetSelected(SelectionEvent e) {
			    ImageLoader loader = new ImageLoader();
			    loader.data = new ImageData[] {model.getImage()};
			    loader.save("swt.bmp", SWT.IMAGE_BMP);
			    File img = new File("swt.bmp");
			    ex.setModel_target(img);
			    ex.addFeatures("track", proprieta.get("track") + "_consegna");
			    FileDialog save = new FileDialog(shell, SWT.SAVE);
			    save.setFilterExtensions(new String[] { "*.pxm" });
			    String fileNameS = save.open();
			    if (fileNameS != null && !fileNameS.isEmpty()) {
				ex.toArchive(fileNameS);
			    }
			    img.deleteOnExit();
			}
		    });
		run();
	
		if (proprieta.containsKey("graphicView")) {
		    if (proprieta.get("graphicView").compareTo("modificabile") == 0) {
			views.add(new EditableGraphicalView(model, shell, this, mode));
		    } else {
			views.add(new GraphicalView(model, shell));
		    }
		}
			
		//		if (proprieta.containsKey("matrix8View")) {
		//			if (proprieta.get("matrix8View").compareTo("modificabile") == 0) {
		//			} else {
		//			}
		//		}
		if (proprieta.containsKey("compressedView")) {
		    if (mode == BW) {
			if (proprieta.get("compressedView").compareTo("modificabile") == 0) {
			    views.add(new CompressedView(model, shell, this, true));
			} else {
			    views.add(new CompressedView(model, shell, this, false));
			}
		    }
		}
		if (proprieta.containsKey("matrixView")) {
		    if (proprieta.get("matrixView").compareTo("modificabile") == 0) {
			views.add(new TableView(shell, model, true, this, mode));
		    } else {
			views.add(new TableView(shell, model, false, this, mode));
		    }
		}
		if (proprieta.containsKey("rectangleView")) {
		    if (proprieta.get("rectangleView").compareTo("modificabile") == 0) {
			views.add(new RectangleView(model, shell, this, mode, true));
		    } else {
			views.add(new RectangleView(model, shell, this, mode, false));
		    }
		}
	
		Iterator<View> iterator = views.iterator();
		while (iterator.hasNext()) {
		    model.addObserver(iterator.next());
		}

    }

    private void initShell() {
		shell.pack();
		shell.setText("Benvenuto in Pixel Manipulator - modalita' LIBERA");
		shell.setSize(width, height);
		shell.setLocation(
				  ((shell.getDisplay().getBounds().width / 2) - (width / 2)),
				  ((shell.getDisplay().getBounds().height / 2) - (height / 2)));
    }

    protected void run() {
		mainShell.setVisible(false);
		shell.open();
		shell.addDisposeListener(new DisposeListener() {
			@Override
			    public void widgetDisposed(DisposeEvent arg0) {
			    mainShell.setVisible(true);
			}
		    });
    }

    public void setImage(ImageData img) {
    	model.setImage(img);
    }

    public void setPixel(int x, int y, RGB rgb) {
		ImageData image = model.getImage();
		try {
		    image.setPixel(x, y, image.palette.getPixel(rgb));
		}
		catch (IllegalArgumentException e) {
		    ;
		}
		model.setImage(image);
    }

    public void zoomIn() {
    	model.setZoom(model.getZoom() + 1);
    }

    public void zoomOut() {
    	model.setZoom(model.getZoom() - 1);
    }

    public void flipH() {
		ImageData img = model.getImage();
		int width = (img.width / 2);
		for (int x = 0; x < width; x++) {
		    for (int y = 0; y < img.height; y++) {
			int tmp = img.getPixel(x, y);
			img.setPixel(x, y, img.getPixel(img.width - 1 - x, y));
			img.setPixel(img.width - 1 - x, y, tmp);
		    }
		}
		model.setImage(img);
    }

    public void flipV() {
		ImageData img = model.getImage();
		int height = (img.height / 2);
		for (int x = 0; x < img.width; x++) {
		    for (int y = 0; y < height; y++) {
			int tmp = img.getPixel(x, y);
			img.setPixel(x, y, img.getPixel(x, img.height - 1 - y));
			img.setPixel(x, img.height - 1 - y, tmp);
		    }
		}
		model.setImage(img);
    }
}
