package it.unimi.dsi.pixelmanipulator.utils;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.HashMap;
//import java.util.Map.Entry;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import org.eclipse.swt.graphics.ImageData;

/**
 * @author Emanuele Galiano
 * @version 1.0.0
 */
public abstract class GenericExercise {
	/**
	 * Buffer size
	 */
	private static final int BUFFER_SIZE = 1000;

	/**
	 * Features of files. Ex: Name, track, description...etc
	 */
	private HashMap<String, String> features = new HashMap<String, String>();

	public HashMap<String, String> getFeatures() {
		return features;
	}

	public void setFeatures(HashMap<String, String> features) {
		this.features = features;
	}

	/**
	 * The first model is used to give an intermediate state exercise that have
	 * to be completed. The second model is the target model.
	 */
	private File model_target;
	private ImageData data;

	public ImageData getData() {
		return data;
	}

	public void setData(ImageData data) {
		this.data = data;
	}

	/**
	 * 
	 * @param name
	 *            name of exercise
	 * @param track
	 *            track of exercise
	 * @param description
	 *            decription of exercise
	 * @param model_target
	 *            this is the model that the student must reach
	 * @param model_under_construction
	 *            this is the model that the teacher give to the student to
	 *            begin the exercise
	 * 
	 */
	public GenericExercise(String name, String track, String description,
			File model_target) {
		this.addFeatures("name", name);
		this.addFeatures("track", track);
		this.addFeatures("description", description);
		this.addFeatures("model_target", model_target.getName());
		this.addExtra();
		this.model_target = model_target;
	}

	/**
	 * This function allow programmer to add features in the constructor method
	 * when extends this class
	 */
	public void addExtra() {
	}

	/**
	 * This function set permanent the model_under_construction so the student
	 * can't modify it
	 */
	public void consegna() {
		this.addFeatures("lock", "true");
	}

	/**
	 * @param path
	 *            the file that contains the exercise
	 */
	public GenericExercise(String path) {

		// costruttore dell'esercizio da file
	}

	/**
	 * @param name
	 *            the name of the feature
	 * @return the feature with the name given
	 */
	public String getFeatures(String name) {
		return this.features.get(name);
	}

	/**
	 * @param name
	 * @param value
	 */
	public void addFeatures(String name, String value) {
		if (this.features.get(name) == null) {
			this.features.put(name, value);
		}
	}

	/**
	 * @return the XML manifest of the exercise
	 */
	public String getXML() {
		StringBufferOutputStream buffer = new StringBufferOutputStream();
		XMLEncoder xmlEncoder = new XMLEncoder(buffer);
		xmlEncoder.writeObject(this.features);
		xmlEncoder.close();
		return buffer.toString();
	}

	/**
	 * @param name
	 *            the name of the exercise file
	 */
	public void toArchive(String name) {
		try {
			String zipName = name.replace(' ', '_');
			String manifest = this.getXML();
			FileOutputStream fos = new FileOutputStream(zipName);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			ZipOutputStream out = new ZipOutputStream(bos);
			this.addFile(out, manifest, "manifesto.xml");
			this.addFile(out, this.model_target, false);
			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	public void setModel_target(File model_target) {
		this.model_target = model_target;
	}
	public GenericExercise() {
	}

	public static GenericExercise open(String filename) {
		GenericExercise ex = new GenericExercise() {
		};
		try {
			ZipFile zf = new ZipFile(filename);
			Enumeration entries = zf.entries();

			while (entries.hasMoreElements()) {

				ZipEntry ze = (ZipEntry) entries.nextElement();
				long size = ze.getSize();
				if (size > 0) {
					BufferedReader br = new BufferedReader(
							new InputStreamReader(zf.getInputStream(ze)));
					if (ze.getName().contains(".xml")) {
						XMLDecoder decoder = new XMLDecoder(
								zf.getInputStream(ze));
						Object o = decoder.readObject();
						decoder.close();
						HashMap<String, String> map = (HashMap<String, String>) o;
						ex.setFeatures(map);
					} else {
						ex.data = new ImageData(zf.getInputStream(ze));
					}
					br.close();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return ex;
	}

	/**
	 * @param out
	 *            is the zip file where we want to add a new file
	 * @param file
	 *            is the file's content we want to add
	 * @param name
	 *            is the name of the new file we are adding to the zip
	 */
	private void addFile(ZipOutputStream out, InputStream file, String name) {
		try {
			int count;
			byte[] data = new byte[BUFFER_SIZE];

			out.putNextEntry(new ZipEntry(name));
			BufferedInputStream in = new BufferedInputStream(file);
			while ((count = in.read(data, 0, BUFFER_SIZE)) != -1) {
				out.write(data, 0, count);
			}
			in.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param out
	 *            is the zip file where we want to add a new file
	 * @param content
	 *            is the file's content we want to add
	 * @param name
	 *            is the name of the new file we are adding to the zip
	 */
	private void addFile(ZipOutputStream out, String content, String name) {
		InputStream in = new ByteArrayInputStream(content.getBytes());
		addFile(out, in, name);
	}

	/**
	 * @param out
	 *            is the zip file where we want to add a new file
	 * @param content
	 *            is the file we want to add
	 * @param delete
	 *            is true if we want to delete the file after added it to the
	 *            zip file
	 */
	private void addFile(ZipOutputStream out, File content, boolean delete) {
		try {
			InputStream in = new FileInputStream(content);
			addFile(out, in, content.getName());
			if (delete)
				content.delete();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
}
