package it.unimi.dsi.pixelmanipulator.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import it.unimi.dsi.pixelmanipulator.controller.PixelController;

public class EditTable {

	private Table table;
	ImageData image = null;
	/**
	 * @uml.property name="controller"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	PixelController controller;
	/**
	 * @uml.property name="can_edit"
	 */
	private boolean can_edit = false;
	/**
	 * @uml.property name="type_of_colors"
	 */
	final int type_of_colors;
	final RGB[][] colors = new RGB[][] {
			new RGB[] { new RGB(255, 255, 255), new RGB(0, 0, 0) },
			new RGB[] { new RGB(255, 255, 255), new RGB(192, 192, 192),
					new RGB(144, 144, 144), new RGB(96, 96, 96),
					new RGB(48, 48, 48), new RGB(0, 0, 0) },
			new RGB[] { new RGB(255, 0, 0), new RGB(0, 255, 0),
					new RGB(0, 0, 255), new RGB(0, 255, 255),
					new RGB(255, 255, 0), new RGB(255, 0, 255) } };
	/**
	 * @uml.property name="rgb6" multiplicity="(0 -1)" dimension="1"
	 */
	final String rgb6[] = new String[] { "<1,0,0>", "<0,1,0>", "<0,0,1>",
			"<0,1,1>", "<1,1,0>", "<1,0,1>" };

	public EditTable(Composite parent, boolean editable,
			PixelController new_controller, int palette) {
		type_of_colors = palette;
		controller = new_controller;
		can_edit = editable;
		table = new Table(parent, SWT.VIRTUAL | SWT.FULL_SELECTION);
	}

	public void init(ImageData new_image) {
		image = new_image;
		if (table.getItemCount() == 0) {
			initTable(image.width, image.height);
			setEvent();
		}
		table.removeAll();
		table.setItemCount(image.height);
	}

	private String getData(int row, int col) {
		RGB color = image.palette.getRGB(image.getPixel(col, row));
		String val = "";
		switch (type_of_colors) {
		case PixelController.BW:
			val = (color.toString().compareTo(
					colors[type_of_colors][1].toString()) == 0) ? "1" : "0";
			break;
		case PixelController.GRAYSCALE:
			for (int i = 0; i < colors[type_of_colors].length; i++) {
				if (colors[type_of_colors][i].toString().compareTo(
						color.toString()) == 0) {
					val = ""+i;
				}
			}
			break;
		case PixelController.RGB:
			val = color.toString();
			break;
		case PixelController.RGB6:
			for (int i = 0; i < colors[type_of_colors].length; i++) {
				if (colors[type_of_colors][i].toString().compareTo(
						color.toString()) == 0) {
					val = rgb6[i];
				}
			}
			break;
		default:
			val = "error";
			break;
		}
		return val;
	}

	private void initTable(int width, int height) {
		table.setVisible(true);
		table.setLinesVisible(true);
		setColumn(width);
		table.addListener(SWT.SetData, new Listener() {
			@Override
			public void handleEvent(Event event) {
				TableItem item = (TableItem) event.item;
				int tableIndex = table.indexOf(item);
				int colCount = table.getColumnCount();
				for (int i = 0; i < colCount; i++) {
					item.setText(i, getData(tableIndex, i));
				}
			}
		});
	}

	private void setColumn(int nCols) {
		for (int c = 0; c < nCols; c++) {
			TableColumn column = new TableColumn(table, SWT.NONE | SWT.LEFT);
			column.setText(String.valueOf(c));
			if (type_of_colors == PixelController.BW) {
				column.setWidth(40);
			} else {
				column.setWidth(150);
			}
			column.setMoveable(false);
			column.setResizable(false);
		}
	}

	private void setEvent() {
		if (can_edit) {
			table.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(MouseEvent e) {
					int col;
					Point clickOn = new Point(e.x, e.y);

					final TableItem item = table.getItem(clickOn);

					if (item != null) {

						col = 0;
						while (!item.getBounds(col).contains(clickOn))
							col++;
						TableEditor editor = new TableEditor(table);
						editor.grabHorizontal = true;
						final Combo combo = new Combo(table, SWT.PUSH);

						switch (type_of_colors) {
						case PixelController.BW:
							combo.add("0");
							combo.add("1");
							break;
						case PixelController.GRAYSCALE:
							combo.add("0");
							combo.add("1");
							combo.add("2");
							combo.add("3");
							combo.add("4");
							combo.add("5");
							break;
						case PixelController.RGB6:
							combo.add("<1,0,0>");
							combo.add("<0,1,0>");
							combo.add("<0,0,1>");
							combo.add("<0,1,1>");
							combo.add("<1,1,0>");
							combo.add("<1,0,1>");
						default:
							break;
						}
						combo.select(0);
						combo.setFocus();
						editor.setEditor(combo, item, col);
						final int c = col;
						combo.addKeyListener(new KeyAdapter() {
							@Override
							public void keyReleased(KeyEvent e) {
								if (e.keyCode == SWT.CR) {
									item.setText(c, combo.getItem(combo
											.getSelectionIndex()));
									controller.setPixel(c, table.indexOf(item),
											colors[type_of_colors][combo
													.getSelectionIndex()]);
									combo.dispose();
								} else if (e.keyCode == SWT.ESC) {
									combo.dispose();
								}
							}
						});
						combo.addFocusListener(new FocusAdapter() {
							@Override
							public void focusLost(FocusEvent e) {
								item.setText(c, combo.getItem(combo
										.getSelectionIndex()));
								controller.setPixel(c, table.indexOf(item),
										colors[type_of_colors][combo
												.getSelectionIndex()]);
								combo.dispose();
							}
						});
					}
				}
			});
		}
	}
}
